/*
 * HTML2TXT
 *
 * Copyright 2000 Matteo Baccan <baccan@infomedia.it>
 * www - http://www.baccan.it
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA (or visit
 * their web site at http://www.gnu.org/).
 *
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <io.h>

#define BUFFER_LEN   4096 // buffer lenght
#define CV_EXITPARA     1 // parameter error
#define CV_EXITOPEN     2 // open error
#define CV_EXITEXIST    3 // file exist

void dfPrintLogo(void);      // logo
void dfPrintInfo(void);      // parameter info
void dfPrintOpenError( const char * InFile ); // open error
void dfPrintExist( const char * OutFile );    // file Exist
void dfConvert( char * Buffer );  // convert string
int dfCheckChar( int iLen,
                 char *Buffer,
                 int iPointer,
                 char *Check,
                 int iCheckLen );

void main(int argc, char *argv[]) {
   FILE *InFile, *OutFile;     // file declaration
   char *RBuffer;              // Read Buffer

   if( argc < 3 ){                                 // check parameter
      dfPrintInfo();
      exit( CV_EXITPARA );
   }// endif

   dfPrintLogo();

   if( access( argv[2], 00 ) >= 0 ){               // file exist
      dfPrintExist( argv[2] );
      exit( CV_EXITEXIST );
   }// endif

   if( (InFile = fopen( argv[1], "r")) == NULL ){  // open file
      dfPrintOpenError( argv[1] );
      exit( CV_EXITOPEN );
   }// endif

   if( (OutFile = fopen( argv[2], "w")) == NULL ){ // create file
      dfPrintOpenError( argv[2] );
      exit( CV_EXITOPEN );
   }// endif

   printf("\n  Reading %s\n", argv[1] );          // Convert

   RBuffer = malloc( BUFFER_LEN );
   while( fgets( RBuffer, BUFFER_LEN, InFile ) != NULL ){
      dfConvert( RBuffer );
      fputs( RBuffer, OutFile );
   }// endwhile
   free( RBuffer );

   printf("\n  OK \n" );

   fclose( InFile );                               // Close File
   fclose( OutFile );

}// end of main


void dfPrintInfo(){
   printf("\n");
   printf("Ŀ\n");
   printf(" HTML2TXT          Converter HTML  to TXT           Version  2.01 \n");
   printf(" Copyright 1997-2000    The Wonderful Team    All Rights Reserved \n");
   printf("  \n");
   printf(" Usage: HTML2TXT <InFile> <OutFile>                               \n");
   printf("                                                                  \n");
   printf(" InFile  = File 2 convert into TXT                                \n");
   printf("                                                                  \n");
   printf(" OutFile = File 2 save                                            \n");
   printf("\n");
}// end of print info

void dfPrintLogo(){
   printf("\n");
   printf("Ŀ\n");
   printf(" HTML2TXT          Converter HTML  to TXT           Version  2.01 \n");
   printf(" Copyright 1997-2000    The Wonderful Team    All Rights Reserved \n");
   printf("\n");
}// end of print logo


void dfPrintOpenError( const char * InFile ){
   printf("\n  Error opening file %s\n", InFile );
}// end of print error


void dfPrintExist( const char * OutFile ){
   printf("\n  Output file %s Exist \n", OutFile );
}// end of print Exist


void dfConvert( char * Buffer ){
   static int iStart=0;
   int  iPos,iPointer;
   int  iLen=strlen( Buffer );
   static int bRem = 0;

   iPos=iPointer=0;

   while( iLen-->0 ){
      iPointer++;

      // REM
      if( dfCheckChar( iLen, Buffer, iPointer, "<!--", 4 ) ){
          iPointer+=3;
          bRem=1;
          continue;
      }
      if( dfCheckChar( iLen, Buffer, iPointer, "-->", 3 ) ){
          iPointer+=2;
          bRem=0;
          continue;
      }
      if( bRem==1 ){
          continue;
      }


      // BlockQuote
      if( dfCheckChar( iLen, Buffer, iPointer, "<BLOCKQUOTE>", 12 ) ){
          Buffer[iPos++]='"';
          iPointer+=11;
          continue;
      }
      if( dfCheckChar( iLen, Buffer, iPointer, "</BLOCKQUOTE>", 13 ) ){
          Buffer[iPos++]='"';
          iPointer+=12;
          continue;
      }

      // LineBreak
      if( dfCheckChar( iLen, Buffer, iPointer, "<BR>", 4 ) ){
          Buffer[iPos++]=0x0d;
          iPointer+=3;
          continue;
      }
      if( dfCheckChar( iLen, Buffer, iPointer, "</BR>", 5 ) ){
          Buffer[iPos++]=0x0d;
          iPointer+=4;
          continue;
      }

      // Citation
      if( dfCheckChar( iLen, Buffer, iPointer, "<CITE>", 6 ) ){
          Buffer[iPos++]='"';
          iPointer+=5;
          continue;
      }
      if( dfCheckChar( iLen, Buffer, iPointer, "</CITE>", 7 ) ){
          Buffer[iPos++]='"';
          iPointer+=6;
          continue;
      }

      // Tab
      if( dfCheckChar( iLen, Buffer, iPointer, "<TD>", 4 ) ){
          Buffer[iPos++]=9;
          iPointer+=3;
          continue;
      }
      if( dfCheckChar( iLen, Buffer, iPointer, "</TD>", 5 ) ){
          Buffer[iPos++]=9;
          iPointer+=4;
          continue;
      }

      // HTML Command Skipper
      if( Buffer[iPointer-1]=='<' ){
         if( Buffer[iPointer]!='\0' ){
            if( Buffer[iPointer]>='a' && Buffer[iPointer]<='z' ) iStart=1;
            if( Buffer[iPointer]>='A' && Buffer[iPointer]<='Z' ) iStart=1;
            if( Buffer[iPointer]=='!'                          ) iStart=1;
            if( Buffer[iPointer]=='/'                          ) iStart=1;
         }
      }
      if( Buffer[iPointer-1]=='>'&& iStart==1 ) {
         iStart=0;
         continue;
      }

      if( iStart==0 ){
         if( Buffer[iPointer-1]=='&' ){
            if( dfCheckChar( iLen, Buffer, iPointer, "&lt;"     , 4 ) ){ Buffer[iPos++]='<'; iPointer+=3; continue; } //4
            if( dfCheckChar( iLen, Buffer, iPointer, "&gt;"     , 4 ) ){ Buffer[iPos++]='>'; iPointer+=3; continue; } //4
            if( dfCheckChar( iLen, Buffer, iPointer, "&amp;"    , 5 ) ){ Buffer[iPos++]='&'; iPointer+=4; continue; } //5
            if( dfCheckChar( iLen, Buffer, iPointer, "&quot;"   , 6 ) ){ Buffer[iPos++]='"'; iPointer+=5; continue; } //6
            if( dfCheckChar( iLen, Buffer, iPointer, "&Aacute;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&Agrave;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&Acirc;"  , 7 ) ){ Buffer[iPos++]=''; iPointer+=6; continue; } //7
            if( dfCheckChar( iLen, Buffer, iPointer, "&Atilde;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&Aring;"  , 7 ) ){ Buffer[iPos++]=''; iPointer+=6; continue; } //7
            if( dfCheckChar( iLen, Buffer, iPointer, "&Auml;"   , 6 ) ){ Buffer[iPos++]=''; iPointer+=5; continue; } //6
            if( dfCheckChar( iLen, Buffer, iPointer, "&AElig;"  , 7 ) ){ Buffer[iPos++]=''; iPointer+=6; continue; } //7
            if( dfCheckChar( iLen, Buffer, iPointer, "&Ccedil;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&Eacute;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&Egrave;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&Ecirc;"  , 7 ) ){ Buffer[iPos++]=''; iPointer+=6; continue; } //7
            if( dfCheckChar( iLen, Buffer, iPointer, "&Euml;"   , 6 ) ){ Buffer[iPos++]=''; iPointer+=5; continue; } //6
            if( dfCheckChar( iLen, Buffer, iPointer, "&Iacute;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&Igrave;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&Icirc;"  , 7 ) ){ Buffer[iPos++]=''; iPointer+=6; continue; } //7
            if( dfCheckChar( iLen, Buffer, iPointer, "&Iuml;"   , 6 ) ){ Buffer[iPos++]=''; iPointer+=5; continue; } //6
            if( dfCheckChar( iLen, Buffer, iPointer, "&ETH;"    , 5 ) ){ Buffer[iPos++]=''; iPointer+=4; continue; } //5
            if( dfCheckChar( iLen, Buffer, iPointer, "&Ntilde;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&Oacute;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&Ograve;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&Ocirc;"  , 7 ) ){ Buffer[iPos++]=''; iPointer+=6; continue; } //7
            if( dfCheckChar( iLen, Buffer, iPointer, "&Otilde;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&Ouml;"   , 6 ) ){ Buffer[iPos++]=''; iPointer+=5; continue; } //6
            if( dfCheckChar( iLen, Buffer, iPointer, "&Oslash;" , 8 ) ){ Buffer[iPos++]='0'; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&Uacute;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&Ugrave;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&Ucirc;"  , 7 ) ){ Buffer[iPos++]=''; iPointer+=6; continue; } //7
            if( dfCheckChar( iLen, Buffer, iPointer, "&Uuml;"   , 6 ) ){ Buffer[iPos++]=''; iPointer+=5; continue; } //6
            if( dfCheckChar( iLen, Buffer, iPointer, "&Yacute;" , 8 ) ){ Buffer[iPos++]='Y'; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&THORN;"  , 7 ) ){ Buffer[iPos++]=''; iPointer+=6; continue; } //7
            if( dfCheckChar( iLen, Buffer, iPointer, "&szlig;"  , 7 ) ){ Buffer[iPos++]=''; iPointer+=6; continue; } //7
            if( dfCheckChar( iLen, Buffer, iPointer, "&aacute;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&agrave;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&acirc;"  , 7 ) ){ Buffer[iPos++]=''; iPointer+=6; continue; } //7
            if( dfCheckChar( iLen, Buffer, iPointer, "&atilde;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&aring;"  , 7 ) ){ Buffer[iPos++]=''; iPointer+=6; continue; } //7
            if( dfCheckChar( iLen, Buffer, iPointer, "&auml;"   , 6 ) ){ Buffer[iPos++]=''; iPointer+=5; continue; } //6
            if( dfCheckChar( iLen, Buffer, iPointer, "&aelig;"  , 7 ) ){ Buffer[iPos++]=''; iPointer+=6; continue; } //7
            if( dfCheckChar( iLen, Buffer, iPointer, "&ccedil;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&eacute;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&egrave;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&ecirc;"  , 7 ) ){ Buffer[iPos++]=''; iPointer+=6; continue; } //7
            if( dfCheckChar( iLen, Buffer, iPointer, "&euml;"   , 6 ) ){ Buffer[iPos++]=''; iPointer+=5; continue; } //6
            if( dfCheckChar( iLen, Buffer, iPointer, "&iacute;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&igrave;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&icirc;"  , 7 ) ){ Buffer[iPos++]=''; iPointer+=6; continue; } //7
            if( dfCheckChar( iLen, Buffer, iPointer, "&iuml;"   , 6 ) ){ Buffer[iPos++]=''; iPointer+=5; continue; } //6
            if( dfCheckChar( iLen, Buffer, iPointer, "&eth;"    , 5 ) ){ Buffer[iPos++]=''; iPointer+=4; continue; } //5
            if( dfCheckChar( iLen, Buffer, iPointer, "&ntilde;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&oacute;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&ograve;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&ocirc;"  , 7 ) ){ Buffer[iPos++]=''; iPointer+=6; continue; } //7
            if( dfCheckChar( iLen, Buffer, iPointer, "&otilde;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&ouml;"   , 6 ) ){ Buffer[iPos++]=''; iPointer+=5; continue; } //6
            if( dfCheckChar( iLen, Buffer, iPointer, "&oslash;" , 8 ) ){ Buffer[iPos++]='0'; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&uacute;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&ugrave;" , 8 ) ){ Buffer[iPos++]=''; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&ucirc;"  , 7 ) ){ Buffer[iPos++]=''; iPointer+=6; continue; } //7
            if( dfCheckChar( iLen, Buffer, iPointer, "&uuml;"   , 6 ) ){ Buffer[iPos++]=''; iPointer+=5; continue; } //6
            if( dfCheckChar( iLen, Buffer, iPointer, "&yacute;" , 8 ) ){ Buffer[iPos++]='Y'; iPointer+=7; continue; } //8
            if( dfCheckChar( iLen, Buffer, iPointer, "&thorn;"  , 7 ) ){ Buffer[iPos++]=''; iPointer+=6; continue; } //7
            if( dfCheckChar( iLen, Buffer, iPointer, "&yuml;"   , 6 ) ){ Buffer[iPos++]=''; iPointer+=5; continue; } //6
            if( dfCheckChar( iLen, Buffer, iPointer, "&reg;"    , 5 ) ){ Buffer[iPos++]='(';
                                                                         Buffer[iPos++]='r';
                                                                         Buffer[iPos++]=')'; iPointer+=4; continue; } //5
            if( dfCheckChar( iLen, Buffer, iPointer, "&copy;"   , 6 ) ){ Buffer[iPos++]='(';
                                                                         Buffer[iPos++]='c';
                                                                         Buffer[iPos++]=')'; iPointer+=5; continue; } //6
            if( dfCheckChar( iLen, Buffer, iPointer, "&trade;"  , 7 ) ){ Buffer[iPos++]='t';
                                                                         Buffer[iPos++]='m'; iPointer+=6; continue; } //7
            if( dfCheckChar( iLen, Buffer, iPointer, "&nbsp;"   , 6 ) ){ Buffer[iPos++]=9;   iPointer+=5; continue; } //6

            // &#number
            if( Buffer[iPointer]=='#' ){
               // May be a Number
               int nCount = 0;
               while( iLen-(nCount+1)>0              && // I have char?
                      Buffer[iPointer+1+nCount]>='0' && // Are number ?
                      Buffer[iPointer+1+nCount]<='9' ){
                  nCount++;
               }

               // If I have number .. try to cenvert it
               if( nCount>0 ){
                  int nDmm  = 0;
                  int nChar = 0;
                  int nMul  = 1;
                  while( nDmm<nCount ){
                     nChar += (Buffer[iPointer+nCount-nDmm]-48)*nMul;
                     printf( "%d\n", Buffer[iPointer+nCount-nDmm]-48 );
                     printf( "%d\n", nMul );
                     nMul  *= 10;
                     nDmm++;
                  }
                  if( nChar>0 ){
                     Buffer[iPos++]=nChar;
                     iPointer+=nDmm+1;
                     continue;
                  }
               }
            }
         }

         if( Buffer[iPointer-1]==0x0d &&
             Buffer[iPointer  ]==0x0d ){
             iPointer++;
             continue;
         }

         Buffer[iPos++]=Buffer[iPointer-1];
      }
   }
   Buffer[iPos++]='\0';
}

// convert string
int dfCheckChar( int iLen,
                 char *Buffer,
                 int iPointer,
                 char *Check,
                 int iCheckLen ){

   int iRet=0;
   if( iLen+1 >= iCheckLen ){
      iPointer--;
      while( *Check!='\0' ){
         if(  Buffer[iPointer]    ==*Check ||
             (Buffer[iPointer]|32)==*Check ){
           iPointer++;
           Check++;
         } else break;
      }
      iRet = (*Check==0 || *Check==13 || *Check==';' || *Check==' ');
   }

   return iRet;
}
